import unittest

from sparktestingbase.testcase import SparkTestingBaseReuse  # type: ignore
from pyspark.sql import SparkSession
from pyspark.ml.feature import StringIndexer, StringIndexerModel

import asyncactions


class AsyncEstimatorActionsTestCase(SparkTestingBaseReuse):
    @classmethod
    def setUpClass(cls):
        SparkTestingBaseReuse.setUpClass()
        cls.spark = SparkSession(cls.sc)

    @classmethod
    def tearDownClass(cls):
        SparkTestingBaseReuse.tearDownClass()
        cls.spark.stop()

    def test_fit_async(self):
        df = self.spark.createDataFrame(["a", "a", "a", "b", "b", "c"], "string")
        indexer = StringIndexer(inputCol="value", outputCol="label")
        f = indexer.fitAsync(df)
        model = f.result()
        self.assertIsInstance(model, StringIndexerModel)



import concurrent.futures
from concurrent.futures.thread import ThreadPoolExecutor

from pyspark.context import SparkContext


def _get_executor(self: SparkContext) -> ThreadPoolExecutor:
    """ Return existing thread pool executor
    or create a new one.
    """
    # This would fail anyway, but
    # we don't want an orphan executor
    if SparkContext._active_spark_context is None:  # type: ignore [attr-defined]
        raise ValueError("No active SparkContext")

    if getattr(SparkContext, "_thread_pool_executor", None) is None:
        import concurrent.futures

        # Make sure that there is only one executor
        with SparkContext._lock:  # type: ignore [attr-defined]
            cores = int(self.getConf().get("spark.driver.cores") or 2)
            # In case of another thread got lock first
            SparkContext._thread_pool_executor = getattr(  # type: ignore [attr-defined]
                SparkContext, "_thread_pool_executor", None
            ) or concurrent.futures.ThreadPoolExecutor(max_workers=cores)

    return SparkContext._thread_pool_executor  # type: ignore [attr-defined]


def stop(self: SparkContext) -> None:
    with SparkContext._lock:  # type: ignore [attr-defined]
        if getattr(SparkContext, "_thread_pool_executor", None):
            SparkContext._thread_pool_executor.shutdown()  # type: ignore [attr-defined]
            SparkContext._thread_pool_executor = None  # type: ignore [attr-defined]
    SparkContext._stop(self)  # type: ignore [attr-defined]


SparkContext._stop = SparkContext.stop  # type: ignore [assignment,attr-defined]
SparkContext.stop = stop  # type: ignore [assignment]
SparkContext._get_executor = _get_executor  # type: ignore [assignment,attr-defined]



import sys
import os
from setuptools import setup, find_packages


setup(
    name="pyspark-asyncactions",
    version="0.0.3",
    packages=find_packages("."),
    url="https://github.com/zero323/pyspark-asyncactions",
    license="Apache 2.0",
    author="zero323",
    author_email="",
    description="A proof of concept asynchronous actions for PySpark using concurent.futures",
    long_description=(
        open("README.rst").read() if os.path.exists("README.rst") else ""
    ),
    install_requires=[],
    classifiers=[
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Development Status :: 3 - Alpha",
        "License :: OSI Approved :: Apache Software License",
    ],
)



from functools import update_wrapper

from pyspark.context import SparkContext
from pyspark.rdd import RDD
from pyspark.sql.dataframe import DataFrame
from pyspark.sql.readwriter import DataFrameWriter


def get_context(self) -> SparkContext:
    return SparkContext._active_spark_context  # type: ignore [attr-defined]


def async_action(f):
    def async_action_(self, *args, **kwargs):
        executor = get_context(self)._get_executor()
        return executor.submit(f, self, *args, **kwargs)

    return async_action_


def patch_async(cls, method, doc, suffix):
    g = getattr(cls, method)
    f = update_wrapper(
        async_action(g),
        g,
        assigned=["__module__", "__annotations__"]
        if hasattr(g, "__annotations__")
        else ["__module__"],
    )
    f.__doc__ = doc
    setattr(cls, "{}{}".format(method, suffix), f)


def patch_all(cls, mapping, suffix: str = "Async"):
    for method, doc in mapping.items():
        patch_async(cls, method, doc, suffix)



from pyspark.ml.base import Estimator
from asyncactions.utils import patch_all

actions = {
    "fit": """Asynchronously fits a model to the input dataset with optional parameters.

        :param dataset: input dataset, which is an instance of :py:class:`pyspark.sql.DataFrame`
        :param params: an optional param map that overrides embedded params. If a list/tuple of
                       param maps is given, this calls fit on each param map and returns a list of
                       models.
        :returns: :py:class:`concurrent.futures.Future` of fitted model(s)
        """
}

patch_all(Estimator, actions)
